package query

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
	"sort"
)

type NodeManagerInfo struct {
	Timestamp int64  `redis:"timestamp" json:"timestamp"`
	Endpoint  string `redis:"endpoint" json:"endpoint"`
}

func (g NodeManagerInfo) TimeStamp() int64 {
	return g.Timestamp
}

func (g NodeManagerInfo) Message() json.RawMessage {
	d, _ := json.Marshal(g)
	return d
}

type nodeManagerQuery struct {
	rdb     *redis.Client
	service common.ServiceType
}

func getNodeManagerQuery(rdb *redis.Client) ServiceQuery {
	return nodeManagerQuery{
		rdb:     rdb,
		service: common.SERVICE_NODE_MANAGER,
	}
}

func (g nodeManagerQuery) ModuleName() string {
	return g.service.String()
}

func (g nodeManagerQuery) List() ([]string, error) {
	all, err := getAll(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	sort.Sort(ServiceInfoList(all))
	var res []string
	for _, v := range all {
		res = append(res, string(v.Message()))
	}
	return res, nil
}

func (g nodeManagerQuery) ServiceInfo(serviceid string) (string, error) {
	info, err := getOne(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	return string(info.Message()), nil
}

func parseInfo(stype common.ServiceType, res *redis.MapStringStringCmd) ServiceInfo {
	var ret ServiceInfo
	switch stype {
	case common.SERVICE_NODE_MANAGER:
		var info NodeManagerInfo
		if err := res.Scan(&info); err != nil {
			return nil
		}
		ret = info
	case common.SERVICE_API_GATEWAY:
		var info GatewayInfo
		if err := res.Scan(&info); err != nil {
			return nil
		}
		ret = info
	case common.SERVICE_BACKEND:
		var info BackendInfo
		if err := res.Scan(&info); err != nil {
			return nil
		}
		ret = info
	case common.SERVICE_SCHEDULER:
		var info SchedulerInfo
		if err := res.Scan(&info); err != nil {
			return nil
		}
		ret = info
	case common.SERVICE_WORKER:
		var info WorkerInfo
		if err := res.Scan(&info); err != nil {
			return nil
		}
		ret = info
	}
	return ret
}

func getAll(rdb *redis.Client, serviceType common.ServiceType) (ServiceInfoList, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var list []ServiceInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		if info := parseInfo(serviceType, res); info != nil {
			list = append(list, info)
		}
	}
	return list, nil
}

func getOne(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (ServiceInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	return parseInfo(serviceType, res), nil
}
