package registry

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
	log "github.com/sirupsen/logrus"
	"sync"
	"time"
)

type RegistryInfo struct {
	ServiceName string          `redis:"service_name" json:"service_name"`
	Timestamp   int64           `redis:"timestamp" json:"timestamp"`
	Endpoint    string          `redis:"endpoint" json:"endpoint"`
	Detail      json.RawMessage `redis:"detail" json:"detail"`
}

type Register interface {
	ServiceType() common.ServiceType
	Endpoint() string
	DetailInfo() (json.RawMessage, error)
}

type Registry struct {
	rdb      *redis.Client
	rw       sync.RWMutex
	register Register
	quit     chan struct{}
}

type RedisConnParam struct {
	Addr     string
	Password string
	DbIndex  int
}

func NewRegistry(redisParam RedisConnParam, register Register) *Registry {
	rdb := redis.NewClient(&redis.Options{
		Addr:     redisParam.Addr,
		Password: redisParam.Password,
		DB:       redisParam.DbIndex,
	})

	return &Registry{
		rdb:      rdb,
		register: register,
		quit:     make(chan struct{}),
	}
}

func (s *Registry) Start() {
	ticker := time.NewTicker(time.Second * 1)
	defer ticker.Stop()

	for {
		select {
		case <-s.quit:
			return
		case <-ticker.C:
			if err := s.registry(s.rdb); err != nil {
				log.WithError(err).Error("registry failed")
			} else {
				ticker.Reset(time.Second * 10)
			}
		}
	}
}

func (s *Registry) Stop() {
	close(s.quit)
}

func (s *Registry) registry(rdb *redis.Client) error {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(s.register.ServiceType()), s.register.Endpoint())
	detail, err := s.register.DetailInfo()
	if err != nil {
		log.WithError(err).Error("get detail info failed")
		return err
	}

	err = rdb.HSet(context.Background(), k, RegistryInfo{
		ServiceName: s.register.ServiceType().String(),
		Timestamp:   time.Now().Unix(),
		Endpoint:    s.register.Endpoint(),
		Detail:      detail,
	}).Err()
	if err != nil {
		log.WithError(err).Error("set register info failed")
	}
	return err
}
